"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBoundary = void 0;
var linear_algebra_1 = require("../linear-algebra");
var int_1 = require("../../mol-data/int");
var boundary_helper_1 = require("./boundary-helper");
var geometry_1 = require("../geometry");
// avoiding namespace lookup improved performance in Chrome (Aug 2020)
var v3set = linear_algebra_1.Vec3.set;
var boundaryHelperCoarse = new boundary_helper_1.BoundaryHelper('14');
var boundaryHelperFine = new boundary_helper_1.BoundaryHelper('98');
function getBoundaryHelper(count) {
    return count > 10000 ? boundaryHelperCoarse : boundaryHelperFine;
}
var p = (0, linear_algebra_1.Vec3)();
function getBoundary(data) {
    var x = data.x, y = data.y, z = data.z, radius = data.radius, indices = data.indices;
    var n = int_1.OrderedSet.size(indices);
    if (n > 250000) {
        var box = geometry_1.Box3D.computeBounding(data);
        return { box: box, sphere: geometry_1.Sphere3D.fromBox3D((0, geometry_1.Sphere3D)(), box) };
    }
    var boundaryHelper = getBoundaryHelper(n);
    boundaryHelper.reset();
    for (var t = 0; t < n; t++) {
        var i = int_1.OrderedSet.getAt(indices, t);
        v3set(p, x[i], y[i], z[i]);
        boundaryHelper.includePositionRadius(p, (radius && radius[i]) || 0);
    }
    boundaryHelper.finishedIncludeStep();
    for (var t = 0; t < n; t++) {
        var i = int_1.OrderedSet.getAt(indices, t);
        v3set(p, x[i], y[i], z[i]);
        boundaryHelper.radiusPositionRadius(p, (radius && radius[i]) || 0);
    }
    var sphere = boundaryHelper.getSphere();
    if (!radius && geometry_1.Sphere3D.hasExtrema(sphere) && n <= sphere.extrema.length) {
        var extrema = [];
        for (var t = 0; t < n; t++) {
            var i = int_1.OrderedSet.getAt(indices, t);
            extrema.push(linear_algebra_1.Vec3.create(x[i], y[i], z[i]));
        }
        geometry_1.Sphere3D.setExtrema(sphere, extrema);
    }
    return { box: boundaryHelper.getBox(), sphere: sphere };
}
exports.getBoundary = getBoundary;
