"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrossLinkColorThemeProvider = exports.CrossLinkColorTheme = exports.getCrossLinkColorThemeParams = exports.CrossLinkColorThemeParams = void 0;
var color_1 = require("../../../mol-util/color");
var param_definition_1 = require("../../../mol-util/param-definition");
var property_1 = require("./property");
var DefaultColor = (0, color_1.Color)(0xCCCCCC);
var Description = 'Colors cross-links by the deviation of the observed distance versus the modeled distance (e.g. modeled / `ihm_cross_link_restraint.distance_threshold`).';
exports.CrossLinkColorThemeParams = {
    domain: param_definition_1.ParamDefinition.Interval([0.5, 1.5], { step: 0.01 }),
    list: param_definition_1.ParamDefinition.ColorList('red-grey', { presetKind: 'scale' }),
};
function getCrossLinkColorThemeParams(ctx) {
    return exports.CrossLinkColorThemeParams; // TODO return copy
}
exports.getCrossLinkColorThemeParams = getCrossLinkColorThemeParams;
function CrossLinkColorTheme(ctx, props) {
    var color;
    var scale = undefined;
    var crossLinkRestraints = ctx.structure && property_1.CrossLinkRestraintProvider.get(ctx.structure).value;
    if (crossLinkRestraints) {
        scale = color_1.ColorScale.create({
            domain: props.domain,
            listOrName: props.list.colors
        });
        var scaleColor_1 = scale.color;
        color = function (location) {
            if (property_1.CrossLinkRestraint.isLocation(location)) {
                var pair = crossLinkRestraints.pairs[location.element];
                if (pair) {
                    return scaleColor_1(property_1.CrossLinkRestraint.distance(pair) / pair.distanceThreshold);
                }
            }
            return DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: CrossLinkColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: scale ? scale.legend : undefined
    };
}
exports.CrossLinkColorTheme = CrossLinkColorTheme;
exports.CrossLinkColorThemeProvider = {
    name: 'cross-link',
    label: 'Cross Link',
    category: "Miscellaneous" /* Misc */,
    factory: CrossLinkColorTheme,
    getParams: getCrossLinkColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.CrossLinkColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure && property_1.CrossLinkRestraint.isApplicable(ctx.structure); },
    ensureCustomProperties: {
        attach: function (ctx, data) { return data.structure ? property_1.CrossLinkRestraintProvider.attach(ctx, data.structure, void 0, true) : Promise.resolve(); },
        detach: function (data) { return data.structure && property_1.CrossLinkRestraintProvider.ref(data.structure, false); }
    }
};
