/**
 * Copyright (c) 2017-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { StructureElement } from './element';
import { SecondaryStructureType } from '../model/types';
declare const StructureProperties: {
    constant: {
        true: StructureElement.Property<boolean>;
        false: StructureElement.Property<boolean>;
        zero: StructureElement.Property<number>;
    };
    atom: {
        key: StructureElement.Property<import("../model").ElementIndex>;
        x: StructureElement.Property<number>;
        y: StructureElement.Property<number>;
        z: StructureElement.Property<number>;
        id: StructureElement.Property<number>;
        occupancy: StructureElement.Property<number>;
        B_iso_or_equiv: StructureElement.Property<number>;
        sourceIndex: StructureElement.Property<number>;
        type_symbol: StructureElement.Property<import("../model/types").ElementSymbol>;
        label_atom_id: StructureElement.Property<string>;
        auth_atom_id: StructureElement.Property<string>;
        label_alt_id: StructureElement.Property<string>;
        label_comp_id: StructureElement.Property<string>;
        auth_comp_id: StructureElement.Property<string>;
        pdbx_formal_charge: StructureElement.Property<number>;
        vdw_radius: StructureElement.Property<number>;
    };
    residue: {
        key: StructureElement.Property<import("../model").ResidueIndex>;
        group_PDB: StructureElement.Property<"ATOM" | "HETATM">;
        label_seq_id: StructureElement.Property<number>;
        auth_seq_id: StructureElement.Property<number>;
        pdbx_PDB_ins_code: StructureElement.Property<string>;
        isNonStandard: StructureElement.Property<boolean>;
        hasMicroheterogeneity: StructureElement.Property<boolean>;
        microheterogeneityCompIds: StructureElement.Property<string[]>;
        secondary_structure_type: StructureElement.Property<SecondaryStructureType>;
        secondary_structure_key: StructureElement.Property<number>;
        chem_comp_type: StructureElement.Property<"other" | "D-peptide linking" | "L-peptide linking" | "D-peptide NH3 amino terminus" | "L-peptide NH3 amino terminus" | "D-peptide COOH carboxy terminus" | "L-peptide COOH carboxy terminus" | "DNA linking" | "RNA linking" | "L-RNA linking" | "L-DNA linking" | "DNA OH 5 prime terminus" | "RNA OH 5 prime terminus" | "DNA OH 3 prime terminus" | "RNA OH 3 prime terminus" | "D-saccharide, beta linking" | "D-saccharide, alpha linking" | "L-saccharide, beta linking" | "L-saccharide, alpha linking" | "L-saccharide" | "D-saccharide" | "saccharide" | "non-polymer" | "peptide linking" | "peptide-like" | "L-gamma-peptide, C-delta linking" | "D-gamma-peptide, C-delta linking" | "L-beta-peptide, C-gamma linking" | "D-beta-peptide, C-gamma linking" | "Ion" | "Lipid">;
    };
    chain: {
        key: StructureElement.Property<import("../model").ChainIndex>;
        label_asym_id: StructureElement.Property<string>;
        auth_asym_id: StructureElement.Property<string>;
        label_entity_id: StructureElement.Property<string>;
    };
    entity: {
        key: StructureElement.Property<import("../model").EntityIndex>;
        id: StructureElement.Property<string>;
        type: StructureElement.Property<"non-polymer" | "polymer" | "macrolide" | "water" | "branched">;
        src_method: StructureElement.Property<"nat" | "man" | "syn">;
        pdbx_description: StructureElement.Property<string[]>;
        formula_weight: StructureElement.Property<number>;
        pdbx_number_of_molecules: StructureElement.Property<number>;
        details: StructureElement.Property<string>;
        pdbx_mutation: StructureElement.Property<string>;
        pdbx_fragment: StructureElement.Property<string>;
        pdbx_ec: StructureElement.Property<string[]>;
        subtype: StructureElement.Property<import("../model/properties/common").EntitySubtype>;
        prd_id: StructureElement.Property<string>;
    };
    unit: {
        id: StructureElement.Property<number>;
        chainGroupId: StructureElement.Property<number>;
        multiChain: StructureElement.Property<boolean>;
        object_primitive: StructureElement.Property<"sphere" | "atomistic" | "gaussian">;
        operator_name: StructureElement.Property<string>;
        model_index: StructureElement.Property<number>;
        model_label: StructureElement.Property<string>;
        model_entry_id: StructureElement.Property<string>;
        hkl: StructureElement.Property<import("../../../mol-math/linear-algebra/3d/vec3").Vec3>;
        spgrOp: StructureElement.Property<number>;
        model_num: StructureElement.Property<number>;
        pdbx_struct_assembly_id: StructureElement.Property<string>;
        pdbx_struct_oper_list_ids: StructureElement.Property<any[]>;
        struct_ncs_oper_id: StructureElement.Property<number>;
    };
    coarse: {
        key: StructureElement.Property<import("../model").ElementIndex>;
        entityKey: StructureElement.Property<import("../model").EntityIndex>;
        x: StructureElement.Property<number>;
        y: StructureElement.Property<number>;
        z: StructureElement.Property<number>;
        asym_id: StructureElement.Property<string>;
        entity_id: StructureElement.Property<string>;
        seq_id_begin: StructureElement.Property<number>;
        seq_id_end: StructureElement.Property<number>;
        sphere_radius: StructureElement.Property<number>;
        sphere_rmsf: StructureElement.Property<number>;
        gaussian_weight: StructureElement.Property<number>;
        gaussian_covariance_matrix: StructureElement.Property<import("../../../mol-math/linear-algebra/3d/mat3").Mat3>;
    };
};
declare type StructureProperties = typeof StructureProperties;
export { StructureProperties };
