"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UniformColorThemeProvider = exports.UniformColorTheme = exports.getUniformColorThemeParams = exports.UniformColorThemeParams = void 0;
var color_1 = require("../../mol-util/color");
var param_definition_1 = require("../../mol-util/param-definition");
var legend_1 = require("../../mol-util/legend");
var mol_util_1 = require("../../mol-util");
var DefaultColor = (0, color_1.Color)(0xCCCCCC);
var Description = 'Gives everything the same, uniform color.';
exports.UniformColorThemeParams = {
    value: param_definition_1.ParamDefinition.Color(DefaultColor),
};
function getUniformColorThemeParams(ctx) {
    return exports.UniformColorThemeParams; // TODO return copy
}
exports.getUniformColorThemeParams = getUniformColorThemeParams;
function UniformColorTheme(ctx, props) {
    var color = (0, mol_util_1.defaults)(props.value, DefaultColor);
    return {
        factory: UniformColorTheme,
        granularity: 'uniform',
        color: function () { return color; },
        props: props,
        description: Description,
        legend: (0, legend_1.TableLegend)([['uniform', color]])
    };
}
exports.UniformColorTheme = UniformColorTheme;
exports.UniformColorThemeProvider = {
    name: 'uniform',
    label: 'Uniform',
    category: "Miscellaneous" /* Misc */,
    factory: UniformColorTheme,
    getParams: getUniformColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.UniformColorThemeParams),
    isApplicable: function (ctx) { return true; }
};
