/**
 * Copyright (c) 2019-2022 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author Áron Samuel Kovács <aron.kovacs@mail.muni.cz>
 */
import { WebGLContext } from '../../mol-gl/webgl/context';
import { RenderTarget } from '../../mol-gl/webgl/render-target';
import { Renderer } from '../../mol-gl/renderer';
import { Scene } from '../../mol-gl/scene';
import { Texture } from '../../mol-gl/webgl/texture';
import { Camera } from '../camera';
import { Helper } from '../helper/helper';
import { StereoCamera } from '../camera/stereo';
import { PostprocessingPass, PostprocessingProps } from './postprocessing';
import { MarkingProps } from './marking';
declare type Props = {
    postprocessing: PostprocessingProps;
    marking: MarkingProps;
    transparentBackground: boolean;
};
declare type RenderContext = {
    renderer: Renderer;
    camera: Camera | StereoCamera;
    scene: Scene;
    helper: Helper;
};
export declare class DrawPass {
    private webgl;
    private readonly drawTarget;
    readonly colorTarget: RenderTarget;
    readonly depthTexture: Texture;
    readonly depthTexturePrimitives: Texture;
    readonly packedDepth: boolean;
    private depthTarget;
    private depthTargetPrimitives;
    private depthTargetVolumes;
    private depthTextureVolumes;
    private depthMerge;
    private copyFboTarget;
    private copyFboPostprocessing;
    private wboit;
    private readonly marking;
    readonly postprocessing: PostprocessingPass;
    private readonly antialiasing;
    get wboitEnabled(): boolean;
    constructor(webgl: WebGLContext, width: number, height: number, enableWboit: boolean);
    reset(): void;
    setSize(width: number, height: number): void;
    private _depthMerge;
    private _renderWboit;
    private _renderBlended;
    private _render;
    render(ctx: RenderContext, props: Props, toDrawingBuffer: boolean): void;
    getColorTarget(postprocessingProps: PostprocessingProps): RenderTarget;
}
export {};
