/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Structure } from '../structure';
import { Result } from '../../../../mol-math/geometry';
import { StructureUniqueSubsetBuilder } from './unique-subset-builder';
import { StructureElement } from '../element';
import { Unit } from '../unit';
import { UnitIndex } from '../element/util';
export interface StructureResult extends Result<StructureElement.UnitIndex> {
    units: Unit[];
}
export declare namespace StructureResult {
    function add(result: StructureResult, unit: Unit, index: StructureElement.UnitIndex, distSq: number): void;
    function create(): StructureResult;
    function copy(out: StructureResult, result: StructureResult): StructureResult;
}
export interface StructureLookup3DResultContext {
    result: StructureResult;
    closeUnitsResult: Result<number>;
    unitGroupResult: Result<UnitIndex>;
}
export declare function StructureLookup3DResultContext(): StructureLookup3DResultContext;
export declare class StructureLookup3D {
    private structure;
    private unitLookup;
    private pivot;
    findUnitIndices(x: number, y: number, z: number, radius: number): Result<number>;
    private findContext;
    find(x: number, y: number, z: number, radius: number, ctx?: StructureLookup3DResultContext): StructureResult;
    private _find;
    findIntoBuilder(x: number, y: number, z: number, radius: number, builder: StructureUniqueSubsetBuilder): void;
    findIntoBuilderIf(x: number, y: number, z: number, radius: number, builder: StructureUniqueSubsetBuilder, test: (l: StructureElement.Location) => boolean): void;
    findIntoBuilderWithRadius(x: number, y: number, z: number, pivotR: number, maxRadius: number, radius: number, eRadius: StructureElement.Property<number>, builder: StructureUniqueSubsetBuilder): void;
    check(x: number, y: number, z: number, radius: number): boolean;
    get boundary(): import("../../../../mol-math/geometry/boundary").Boundary;
    constructor(structure: Structure);
}
